Attribute VB_Name = "mEffects"
Option Explicit

'Types
    Private Type tBitmapInfoHeader
        Size As Long
        W As Long
        H As Long
        Planes As Integer
        ColorDepth As Integer
        Compression As Long
        SizeImage As Long
        xPPM As Long
        yPPM As Long
        UsedColosr As Long
        ImportantColors As Long
    End Type
    
    Public Type tColorRGBA
        B As Byte
        G As Byte
        R As Byte
        A As Byte
    End Type
    
    Private Type tBitmap
        Type As Long
        W As Long
        H As Long
        WidthBytes As Long
        Planes As Integer
        ColorDepth As Integer
        Bits As Long
    End Type
    
    Private Type tBitmapInfo
        Header As tBitmapInfoHeader
        Colors As tColorRGBA
    End Type

'Declares
    Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal iDC As Long) As Long
    Private Declare Function DeleteDC Lib "gdi32" (ByVal iDC As Long) As Long
    
    Private Declare Function GetObjectAPI Lib "gdi32" Alias "GetObjectA" (ByVal Object As Long, ByVal Count As Long, Target As Any) As Long
    
    Private Declare Function GetDIBits Lib "gdi32" (ByVal iDC As Long, ByVal iBitmap As Long, ByVal iStartScan As Long, ByVal iNumScans As Long, iBits As Any, iBitmapInfo As tBitmapInfo, ByVal iUsage As Long) As Long
    Private Declare Function SetDIBits Lib "gdi32" (ByVal iDC As Long, ByVal iBitmap As Long, ByVal iStartScan As Long, ByVal iNumScans As Long, iBits As Any, iBitmapInfo As tBitmapInfo, ByVal iUsage As Long) As Long

'Variables
    Private Data() As tColorRGBA
    
    Private TempBitmapInfo As tBitmapInfo
    Private TempBitmap As tBitmap
    Private TempDC As Long
    
    Private SourceDC As Long
    Private SourceHandle As Long
    
    Private Locked As Boolean

Public Function CreateColor(R As Byte, G As Byte, B As Byte) As tColorRGBA
    With CreateColor
        'Apply color
        .R = R
        .G = G
        .B = B
    End With
End Function

Public Sub Release()
    'Release DC from target
    DeleteDC TempDC
    TempDC = 0
    
    'Unlock
    Locked = False
End Sub

Public Sub SetPicture(iSourceDC As Long, iSourceHandle As Long)
    Dim Temp As Long
    
    'If already locked release first
    If Locked Then: Release
    
    'Get parameters
    SourceDC = iSourceDC
    SourceHandle = iSourceHandle
    
    'Get DC and attach to target
    TempDC = CreateCompatibleDC(iSourceDC)
    If TempDC = 0 Then: Exit Sub
    Temp = GetObjectAPI(iSourceHandle, Len(TempBitmap), TempBitmap)
    If Temp = 0 Then: Exit Sub
    
    'Setup bitmap info
    With TempBitmapInfo.Header
        'Internal setup
        .ColorDepth = TempBitmap.ColorDepth
        .Planes = TempBitmap.Planes
        .Size = 40
        
        'Get size
        .W = TempBitmap.W
        .H = TempBitmap.H
        
        'Create data buffer
        ReDim Data((.W * .H) - 1)
    End With
    
    'Get bitmap data (note that it's top-down)
    Temp = GetDIBits(TempDC, iSourceHandle, 0, TempBitmap.H, Data(0), TempBitmapInfo, 0)
    If Temp = 0 Then: Exit Sub
    
    'Lock class
    Locked = True
End Sub

Public Sub Brighten(iAmountR As Byte, iAmountG As Byte, iAmountB As Byte)
    Dim A As Long
    
    'Security check
    If Not Locked Then: Exit Sub
    
    'Edit data
    Dim TempR As Integer
    Dim TempG As Integer
    Dim TempB As Integer
    
    For A = 0 To UBound(Data)
        With Data(A)
            'Get color
            TempR = CInt(.R) + iAmountR
            TempG = CInt(.G) + iAmountG
            TempB = CInt(.B) + iAmountB
            
            'Overflow checks
            If TempR > 255 Then: TempR = 255
            If TempG > 255 Then: TempG = 255
            If TempB > 255 Then: TempB = 255
            
            'Set color
            .R = CByte(TempR)
            .G = CByte(TempG)
            .B = CByte(TempB)
        End With
    Next
End Sub

Public Sub ReplaceColor(iFromColor As tColorRGBA, iToColor As tColorRGBA)
    Dim A As Long
    
    'Security check
    If Not Locked Then: Exit Sub
    
    'Edit data
    Dim TempR As Integer
    Dim TempG As Integer
    Dim TempB As Integer
    
    For A = 0 To UBound(Data)
        With Data(A)
            'Replace color if matches
            If .R = iFromColor.R And .G = iFromColor.G And .B = iFromColor.B Then: Data(A) = iToColor
        End With
    Next
End Sub

Public Sub Darken(iAmountR As Byte, iAmountG As Byte, iAmountB As Byte)
    Dim A As Long
    
    'Security check
    If Not Locked Then: Exit Sub
    
    'Edit data
    Dim TempR As Integer
    Dim TempG As Integer
    Dim TempB As Integer
    
    For A = 0 To UBound(Data)
        With Data(A)
            'Get color
            TempR = CInt(.R) - iAmountR
            TempG = CInt(.G) - iAmountG
            TempB = CInt(.B) - iAmountB
            
            'Overflow checks
            If TempR < 0 Then: TempR = 0
            If TempG < 0 Then: TempG = 0
            If TempB < 0 Then: TempB = 0
            
            'Set color
            .R = CByte(TempR)
            .G = CByte(TempG)
            .B = CByte(TempB)
        End With
    Next
End Sub
Public Sub Apply()
    'Security check
    If Not Locked Then: Exit Sub
    
    'Write data back
    SetDIBits SourceDC, SourceHandle, 0, TempBitmap.H, Data(0), TempBitmapInfo, 0
End Sub

